==> raqdevil-0.9.6.tar.gz: raqdevil-0.9.6/cce-handlers-1.0/sbin/update.php <==
#!(prefix)/bin/php -q
<?php

set_time_limit(0);

define('VERSION', '0.9.6');
$installed = array();
$progress = 0;

function parseCSV($string) {
	$inquote = $rrefd1 = $rrefd2 = 0;
	$ret = array();

	for ($i = 0; $i < strlen($string); $i++) {
		if ($string[$i] == "\n") {
			$rrefd2 = 0;
			$rrefd1++;
			continue;
		}
		if ($string[$i] == '"' && $inquote == 0) {
			$inquote = 1;
			continue;
		} elseif ($string[$i] == '"' && $inquote == 1) {
			$inquote = 0;
			continue;
		}
		if ($string[$i] == ',' && !$inquote) {
			$rrefd2++;
			continue;
		}

		$ret[$rrefd1][$rrefd2] .= $string[$i];
	}

	return $ret;
}

function readHTTPDigestAuthenticatedFile($host,$file,$username,$password,$line,$num)
{
   global $total_install_size;
   global $current_install_size;
   global $total_packages;
   global $progress;

   if (!$fp=fsockopen($host,80, $errno, $errstr, 15))
       return false;
      
   //first do the non-authenticated header so that the server
   //sends back a 401 error containing its nonce and opaque
   $out = "GET $file HTTP/1.1\r\n";
       $out .= "Host: $host\r\n";
       $out .= "Connection: Close\r\n\r\n";

     fwrite($fp, $out);

   //read the reply and look for the WWW-Authenticate element
   while (!feof($fp))
   {
       $line=fgets($fp, 512);
      
       if (strpos($line,"WWW-Authenticate:")!==false)
	   $authline=trim(substr($line,18));
   }
  
   fclose($fp);
      
   //split up the WWW-Authenticate string to find digest-realm,nonce and opaque values
   //if qop value is presented as a comma-seperated list (e.g auth,auth-int) then it won't be retrieved correctly
   //but that doesn't matter because going to use 'auth' anyway
   $authlinearr=explode(",",$authline);
   $autharr=array();
  
   foreach ($authlinearr as $el)
   {
       $elarr=explode("=",$el);
       //the substr here is used to remove the double quotes from the values
       $autharr[trim($elarr[0])]=substr($elarr[1],1,strlen($elarr[1])-2);
   }
  
   //these are all the vals required from the server
   $nonce=$autharr['nonce'];
   $opaque=$autharr['opaque'];
   $drealm=$autharr['Digest realm'];
  
   //calculate the hashes of A1 and A2 as described in RFC 2617
   $a1="$username:$drealm:$password";$a2="GET:$file";
   $ha1=md5($a1);$ha2=md5($a2);
 
   $concat = $ha1.':'.$nonce.':'.$ha2;
   $response=md5($concat);
 
   //put together the Authorization Request Header
   $out = "GET $file HTTP/1.1\r\n";
   $out .= "Host: $host\r\n";
   $out .= "Connection: Close\r\n";
   $out .= "Authorization: Digest username=\"$username\", realm=\"$drealm\", uri=\"$file\", nonce=\"$nonce\", response=\"$response\"\r\n\r\n";

   if (!$fp=fsockopen($host,80, $errno, $errstr, 5))
       return false;
  
   fwrite($fp, $out);
 
   /* Write file directly to temp file so that we don't exceed memory limits */ 
   $fn = tempnam("/tmp", "rdupdate");
   $fd1 = fopen($fn, 'w');

   $have_trashed_headers = 0;

   while (!feof($fp))
   {
       $str = fread($fp, 4096);
       if (!$have_trashed_headers) {
	$have_trashed_headers = 1;
	$str = substr($str, strpos($str, "\r\n\r\n") + 4);
       }
       fwrite ($fd1, $str);
       $current_install_size += strlen($str);
       $progress = floor(($current_install_size * 100) / $total_install_size);
       updateStatus("title: Update package $num of $total_packages\nmessage: Downloading package ${line[1]}-${line[3]} (".(int)($current_install_size/1024)."KB of ".(int)($total_install_size/1024)."KB)\nprogress: $progress", 1) ;
   }
  
   fclose($fp);

   return $fn;
}

function restore($name, $num) {
	global $installed;
	global $total_packages;

	if (count($installed) != 0) {
		foreach ($installed as $pkg) {
			 print_r($pkg);
			`/usr/sbin/pkg_delete -f $pkg[1]`;
			`/usr/sbin/pkg_add -f (prefix)/raqdevil/package-backups/$pkg[0].tbz`;
		}
	}

	`/bin/rm -f /tmp/.rd_update.lock`;

	updateStatus("title: Failed updating package $num of $total_packages\nmessage: Failed to add package $name. Please report this error at http://www.raqdevil.com/ immediately!\nisNoRefresh: true\nbackUrl: /base/swupdate/splashSoftware.php\n");

	die('Failed gracefully.');
}

function add_package($line, $num) {
	global $installed;
	global $total_packages;
	global $progress;

	$package = "${line[1]}-${line[3]}";
	$url = $line[4];
	$new = $line[8];
	$op = $line[8] ? '' : "${line[1]}-${line[2]}";
	$cksum = $line[5];

	//echo "adding package $url\n";

	$aurl = parse_url($url);

	if (!$new) {
		updateStatus("title: Update package $num of $total_packages\nmessage: Backing up old version of ${line[1]}\nprogress: $progress", 1) ;
		$i = count($installed);
		$installed[$i][0] = $op;
		$installed[$i][1] = substr($aurl['path'], 1);
		`/bin/mkdir -p (prefix)/raqdevil/package-backups`;
		`/usr/sbin/pkg_create -b $op (prefix)/raqdevil/package-backups/$op.tbz`;
		updateStatus("title: Update package $num of $total_packages\nmessage: Removing old version of ${line[1]}\nprogress: $progress", 1) ;
		`/usr/sbin/pkg_delete -f $op`;
	}

	$fn = readHTTPDigestAuthenticatedFile($aurl['host'], $aurl['path'], "raqdevil_opensource", "mooBurgers", $line, $num);

	$tofile = '/tmp/'.basename($aurl['path']);
	`/bin/mv $fn $tofile`;
	$poo = md5_file($fn);

	if ($cksum != md5_file($tofile) && !$new) {
		// echo "Invalid checksum. Fail gracefully and restore all upgraded packages.";
		restore($line[1], $num);
	}

	updateStatus("title: Update package $num of $total_packages\nmessage: Adding new version of ${line[1]}\nprogress: $progress", 1) ;
	system("/usr/sbin/pkg_add $f", $rv);

	if ($rv != 0 && !$new) {
		// echo "Couldn't add package. Fail gracefully and restore all upgraded packages.";
		restore($line[1], $num);
	}

	unlink($f);
}

function updateStatus($str)
{
	global $fd;

	fseek($fd, 0);
	fputs($fd, $str);
	fflush($fd);
	ftruncate($fd, strlen($str));
}

$statusFile = $argv[2];

$fd = fopen($argv[1], 'r');
if (!$fd) {
	die ("I can't open the lock file, so I can't rely on actually outputting errors to a file :(");
}
$updates_a = unserialize(base64_decode(fread($fd, filesize($argv[1]))));
fclose($fd);

$fd = fopen($statusFile, 'w');
if (!fd) {
	die("WTF?");
}

$total_install_size = 0;
$total_packages = count($updates_a);

foreach ($updates_a as $line) {
	$total_install_size += $line[6];
}

$current_install_size = 0;

foreach($updates_a as $i => $line) {
	add_package($line, $i + 1);
}

updateStatus("redirectUrl: /base/swupdate/splashSoftware.php\n");
`rm -f /tmp/.rd_update.lock`;
?>

==> raqdevil-1.0.tar.gz: raqdevil-1.0/cce-handlers-1.0/sbin/update.php <==
#!(prefix)/bin/php -q
<?php

set_time_limit(0);

define('VERSION', '1.0');
$installed = array();
$progress = 0;

function parseCSV($string) {
	$inquote = $rrefd1 = $rrefd2 = 0;
	$ret = array();

	for ($i = 0; $i < strlen($string); $i++) {
		if ($string[$i] == "\n") {
			$rrefd2 = 0;
			$rrefd1++;
			continue;
		}
		if ($string[$i] == '"' && $inquote == 0) {
			$inquote = 1;
			continue;
		} elseif ($string[$i] == '"' && $inquote == 1) {
			$inquote = 0;
			continue;
		}
		if ($string[$i] == ',' && !$inquote) {
			$rrefd2++;
			continue;
		}

		$ret[$rrefd1][$rrefd2] .= $string[$i];
	}

	return $ret;
}

function readHTTPDigestAuthenticatedFile($host,$file,$username,$password,$line,$num)
{
   global $total_install_size;
   global $current_install_size;
   global $total_packages;
   global $progress;

   if (!$fp=fsockopen($host,80, $errno, $errstr, 15))
       return false;
      
   //first do the non-authenticated header so that the server
   //sends back a 401 error containing its nonce and opaque
   $out = "GET $file HTTP/1.1\r\n";
       $out .= "Host: $host\r\n";
       $out .= "Connection: Close\r\n\r\n";

     fwrite($fp, $out);

   //read the reply and look for the WWW-Authenticate element
   while (!feof($fp))
   {
       $line=fgets($fp, 512);
      
       if (strpos($line,"WWW-Authenticate:")!==false)
	   $authline=trim(substr($line,18));
   }
  
   fclose($fp);
      
   //split up the WWW-Authenticate string to find digest-realm,nonce and opaque values
   //if qop value is presented as a comma-seperated list (e.g auth,auth-int) then it won't be retrieved correctly
   //but that doesn't matter because going to use 'auth' anyway
   $authlinearr=explode(",",$authline);
   $autharr=array();
  
   foreach ($authlinearr as $el)
   {
       $elarr=explode("=",$el);
       //the substr here is used to remove the double quotes from the values
       $autharr[trim($elarr[0])]=substr($elarr[1],1,strlen($elarr[1])-2);
   }
  
   //these are all the vals required from the server
   $nonce=$autharr['nonce'];
   $opaque=$autharr['opaque'];
   $drealm=$autharr['Digest realm'];
  
   //calculate the hashes of A1 and A2 as described in RFC 2617
   $a1="$username:$drealm:$password";$a2="GET:$file";
   $ha1=md5($a1);$ha2=md5($a2);
 
   $concat = $ha1.':'.$nonce.':'.$ha2;
   $response=md5($concat);
 
   //put together the Authorization Request Header
   $out = "GET $file HTTP/1.1\r\n";
   $out .= "Host: $host\r\n";
   $out .= "Connection: Close\r\n";
   $out .= "Authorization: Digest username=\"$username\", realm=\"$drealm\", uri=\"$file\", nonce=\"$nonce\", response=\"$response\"\r\n\r\n";

   if (!$fp=fsockopen($host,80, $errno, $errstr, 5))
       return false;
  
   fwrite($fp, $out);
 
   /* Write file directly to temp file so that we don't exceed memory limits */ 
   $fn = tempnam("/tmp", "rdupdate");
   $fd1 = fopen($fn, 'w');

   $have_trashed_headers = 0;

   while (!feof($fp))
   {
       $str = fread($fp, 4096);
       if (!$have_trashed_headers) {
	$have_trashed_headers = 1;
	$str = substr($str, strpos($str, "\r\n\r\n") + 4);
       }
       fwrite ($fd1, $str);
       $current_install_size += strlen($str);
       $progress = floor(($current_install_size * 100) / $total_install_size);
       updateStatus("title: Update package $num of $total_packages\nmessage: Downloading package ${line[1]}-${line[3]} (".(int)($current_install_size/1024)."KB of ".(int)($total_install_size/1024)."KB)\nprogress: $progress", 1) ;
   }
  
   fclose($fp);

   return $fn;
}

function restore($name, $num) {
	global $installed;
	global $total_packages;

	if (count($installed) != 0) {
		foreach ($installed as $pkg) {
			 print_r($pkg);
			`/usr/sbin/pkg_delete -f $pkg[1]`;
			`/usr/sbin/pkg_add -f (prefix)/raqdevil/package-backups/$pkg[0].tbz`;
		}
	}

	`/bin/rm -f /tmp/.rd_update.lock`;

	updateStatus("title: Failed updating package $num of $total_packages\nmessage: Failed to add package $name. Please report this error at http://www.raqdevil.com/ immediately!\nisNoRefresh: true\nbackUrl: /base/swupdate/splashSoftware.php\n");

	die('Failed gracefully.');
}

function add_package($line, $num) {
	global $installed;
	global $total_packages;
	global $progress;

	$package = "${line[1]}-${line[3]}";
	$url = $line[4];
	$new = $line[8];
	$op = $line[8] ? '' : "${line[1]}-${line[2]}";
	$cksum = $line[5];

	//echo "adding package $url\n";

	$aurl = parse_url($url);

	if (!$new) {
		updateStatus("title: Update package $num of $total_packages\nmessage: Backing up old version of ${line[1]}\nprogress: $progress", 1) ;
		$i = count($installed);
		$installed[$i][0] = $op;
		$installed[$i][1] = substr($aurl['path'], 1);
		`/bin/mkdir -p (prefix)/raqdevil/package-backups`;
		`/usr/sbin/pkg_create -j -b $op (prefix)/raqdevil/package-backups/$op.tbz`;
		updateStatus("title: Update package $num of $total_packages\nmessage: Removing old version of ${line[1]}\nprogress: $progress", 1) ;
		`/usr/sbin/pkg_delete -f $op`;
	}

	$fn = readHTTPDigestAuthenticatedFile($aurl['host'], $aurl['path'], "raqdevil_opensource", "mooBurgers", $line, $num);

	$tofile = '/tmp/'.basename($aurl['path']);
	`/bin/mv $fn $tofile`;
	$poo = md5_file($tofile);

	if ($cksum != md5_file($tofile) && !$new) {
		// echo "Invalid checksum. Fail gracefully and restore all upgraded packages.";
		restore($line[1], $num);
	}

	updateStatus("title: Update package $num of $total_packages\nmessage: Adding new version of ${line[1]}\nprogress: $progress", 1) ;
	system("/usr/sbin/pkg_add $tofile", $rv);

	if ($rv != 0 && !$new) {
		// echo "Couldn't add package. Fail gracefully and restore all upgraded packages.";
		restore($line[1], $num);
	}

	unlink($tofile);
}

function updateStatus($str)
{
	global $fd;

	fseek($fd, 0);
	fputs($fd, $str);
	fflush($fd);
	ftruncate($fd, strlen($str));
}

$statusFile = $argv[2];

$fd = fopen($argv[1], 'r');
if (!$fd) {
	die ("I can't open the lock file, so I can't rely on actually outputting errors to a file :(");
}
$updates_a = unserialize(rawurldecode(fread($fd, filesize($argv[1]))));
fclose($fd);

$fd = fopen($statusFile, 'w');
if (!fd) {
	die("No status file descriptor");
}

$total_install_size = 0;
$total_packages = count($updates_a);

foreach ($updates_a as $line) {
	$total_install_size += $line[6];
}

$current_install_size = 0;

foreach($updates_a as $i => $line) {
	add_package($line, $i + 1);
}

updateStatus("redirectUrl: /base/swupdate/splashSoftware.php\n");
`/bin/rm -f /tmp/.rd_update.lock`;
?>
